/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.edg;

import org.infinispan.Cache;
import org.infinispan.config.CacheLoaderManagerConfig;
import org.infinispan.config.Configuration;
import org.infinispan.config.Configuration.CacheMode;
import org.infinispan.config.GlobalConfiguration;
import org.infinispan.eviction.EvictionStrategy;
import org.infinispan.loaders.cluster.ClusterCacheLoaderConfig;
import org.infinispan.manager.DefaultCacheManager;

/* ran with JVM args
 * -Dlog4j.configuration=file:///.../src/main/resources/log4j.properties 
 * -Djava.net.preferIPv4Stack=true
 * 
 */
public class NonStartedCacheAccessProblem {
   
   public static void main(String[] args) throws Exception {
      GlobalConfiguration gcfg1 = GlobalConfiguration.getClusteredDefault();
      gcfg1.getTransportProperties().setProperty("configurationFile", "jgroups-udp1.xml");
      GlobalConfiguration gcfg2 = GlobalConfiguration.getClusteredDefault();
      gcfg1.getTransportProperties().setProperty("configurationFile", "jgroups-udp2.xml");

      final DefaultCacheManager dcm1 = new DefaultCacheManager(gcfg1, createConfiguration());
      final DefaultCacheManager dcm2 = new DefaultCacheManager(gcfg2, createConfiguration());

      Thread t1 = new Thread() {
         public void run() {
            dcm1.getCache();
         };
      };

      Thread t2 = new Thread() {
         public void run() {
            Cache<String, String> c = dcm2.getCache();
            c.put("a", "a"); 
            c.get("a");
         };
      };

      t1.start();
      t2.start();

      t1.join();
      t2.join();
   }

   private static Configuration createConfiguration() {
      long distSyncTimeout = 600000;
      long lockTimeout = 10000;
      long replTimeout = 15000;
      boolean doStateTransfer = false;
      long topologyUpdateTimeout = 60000;

      Configuration topologyCacheConfig = new Configuration();
      topologyCacheConfig.fluent().stateRetrieval().fetchInMemoryState(false);
      topologyCacheConfig.setCacheMode(CacheMode.REPL_SYNC);
      topologyCacheConfig.setSyncReplTimeout(replTimeout);
      topologyCacheConfig.setLockAcquisitionTimeout(lockTimeout);
      topologyCacheConfig.setEvictionStrategy(EvictionStrategy.NONE); // No eviction
      topologyCacheConfig.setExpirationLifespan(-1); // No maximum lifespan
      topologyCacheConfig.setExpirationMaxIdle(-1); // No maximum idle time
      CacheLoaderManagerConfig loaderConfigs = new CacheLoaderManagerConfig();
      ClusterCacheLoaderConfig clusterLoaderConfig = new ClusterCacheLoaderConfig();
      clusterLoaderConfig.setRemoteCallTimeout(replTimeout);
      loaderConfigs.addCacheLoaderConfig(clusterLoaderConfig);
      topologyCacheConfig.setCacheLoaderManagerConfig(loaderConfigs);
      return topologyCacheConfig;
   }
}
